
local AutoRoller = LibStub('AceAddon-3.0'):NewAddon('AutoRoller', 'AceEvent-3.0', 'AceConsole-3.0')
_G.AutoRoller = AutoRoller

local L = LibStub('AceLocale-3.0'):GetLocale('AutoRoller', true)

local chatFrame = DEFAULT_CHAT_FRAME
local format = string.format

---------------   initialize   ---------------

-- action := {0 : pass, 1 : need, 2 : greed, 3 : nothing}

local defaults = {
	profile = {
		enabled = true,
		chat = true,
      disableBoPConfirm = false,
		rules = {
         {
            enabled = true,
            action = 2,
            logic = 'boe'
         },  -- 1
         {
            enabled = true,
            action = 3,
            logic = 'bop'
         },
         {
            enabled = true,
            action = 3,
            logic = [[itype == 'Recipe']]
         },
      },
	},
}

AutoRoller.rollType = {[0] = PASS, [1] = NEED, [2] = GREED, [3] = NONE}
AutoRoller.ruleNameLUT = {} --lookup table, for each new rule add [N],['rule'..N] = 'rule'..N, N

function AutoRoller:OnInitialize()
   self.db = LibStub('AceDB-3.0'):New('AutoRollerDB', defaults, 'Default')

   self.db.RegisterCallback(self, 'OnProfileChanged', 'OnEnable')
   self.db.RegisterCallback(self, 'OnProfileCopied', 'OnEnable')
   self.db.RegisterCallback(self, 'OnProfileReset', 'OnEnable')

	self:RegisterOptions()
end

function AutoRoller:OnEnable()
   local profile = self.db.profile
   if not profile.enabled then
      return self:Disable()
   end

   local lut,n = self.ruleNameLUT
   self.optionTable.args.rules.args = {}
   for i=1,#(profile.rules) do
      n='rule'..i
      lut[i],lut[n] = n,i
      self:BuildRuleOption(i, n)
   end

   self:RegisterEvent('CONFIRM_LOOT_ROLL')
   return self:RegisterEvent('START_LOOT_ROLL')
end

function AutoRoller:OnDisable()
end

---------------   Core   ---------------

local tags = {
   --hard coded values
   poor = 0,
   common = 1,
   uncommon = 2,
   rare = 3,
   epic = 4,
   legendary = 5,
   artifact = 6,
   heirloom = 7,

   --mutable per item
   raid = false, --(GetNumRaidMembers() > 0),
	party = true, --not raid
	combat = false, --InCombatLockdown()
	quality = 2,
	itype = 'Armor', --'Armor', 'Consumable', 'Container', 'Gem', 'Key', 'Miscellaneous', 'Money', 'Reagent', 'Recipe', 'Projectile', 'Quest', 'Quiver', 'Trade Goods', 'Weapon'
   subtype = 'Plate', --http://www.wowwiki.com/ItemType
   ilvl = 213,
   reqlvl = 80,
   name = 'Item of Lewting',
   id = 123456,
   boe = true,
   bop = false,
}

setmetatable(tags, {__index = _G})

local function GetItemID(link)
   local _,_,id = link:find('item:(.-):')
   return tonumber(id)
end

local function pcallLogic(logic)
   local func, error = loadstring('return (('..logic..') or false) and true') -- make sure return is true or false
   if not func then
      return false, error
   end
   setfenv(func, tags)
   return pcall(func)
end

local function EvaluateRules(rules)
   local success, val
   for _,rule in ipairs(rules) do
      if rule.enabled then
         success, val = pcallLogic(rule.logic)
         if success and val then
            return rule.action
         end
      end
   end

   return 3 --do nothing
end

function AutoRoller:ValidateLogic(info, value)
   local success, error = pcallLogic(value)
   return success or error
end

function AutoRoller:START_LOOT_ROLL(event, rollID)
   local profile = self.db.profile
   if not profile.enabled then return end

   local link,_ = GetLootRollItemLink(rollID)

   _, tags.name, _, tags.quality, tags.bop = GetLootRollItemInfo(rollID)
   _,_,_, tags.ilvl, tags.reqlvl,  tags.itype, tags.subtype = GetItemInfo(link) -- cant use name (item must be in inventory)

   tags.raid = GetNumRaidMembers() > 0
	tags.party = not tags.raid
	tags.combat = InCombatLockdown()
	tags.id = GetItemID(link)
   tags.boe = not tags.bop

   local roll = EvaluateRules(self.db.profile.rules)
   if roll == 3 then return end

   if profile.chat then
      local msg
      if roll == 0 then
         msg = LOOT_ROLL_PASSED
      elseif roll == 1 then
         msg = LOOT_ROLL_NEED
      else
         msg = LOOT_ROLL_GREED
      end
      chatFrame:AddMessage(msg:format("|cff00AA00AutoRoller|r", link))
   end

   return RollOnLoot(rollID, roll)
end

function AutoRoller:CONFIRM_LOOT_ROLL(event, id, roll)
   if not self.db.profile.disableBoPConfirm then return end

   ConfirmLootRoll(id, roll)
   StaticPopup_Hide('CONFIRM_LOOT_ROLL') --may not be necessary
end

local function EnabledColor(enabled)
   if enabled then
      return 1, 1, 1, 1, 1, 1
   else
      return .7, .7, .7, .7, .7, .7
   end
end

AutoRoller.dataobj = LibStub:GetLibrary('LibDataBroker-1.1'):NewDataObject('AutoLauncher', {
   type = 'launcher',
   icon = 'Interface\\Buttons\\UI-GroupLoot-Dice-Up',
   tocname = 'AutoRoller',
   label = 'AutoRoller',
   OnTooltipShow = function(tooltip)
      tooltip:AddLine('AutoRoller')
      local profile = AutoRoller.db.profile
      local enabled, rollType = profile.enabled, AutoRoller.rollType
      local red = enabled and 0 or 1
      tooltip:AddLine(enabled and ENABLE or ADDON_DISABLED , red, 1 - red, 0)
      enabled = profile.chat
      red = enabled and 0 or 1
      tooltip:AddDoubleLine(CHAT, enabled and ENABLE or ADDON_DISABLED , 0, 1, 0, red, 1 - red, 0)
      enabled = profile.disableBoPConfirm
      red = enabled and 0 or 1
      tooltip:AddDoubleLine(L["Hide BoP Confirmation"], enabled and ENABLE or ADDON_DISABLED , 0, 1, 0, red, 1 - red, 0)
      for i,rule in pairs(profile.rules) do
         tooltip:AddDoubleLine(i..'. '..rule.logic,  rollType[rule.action], EnabledColor(rule.enabled))
      end
   end,
})
