local self = AutoRoller
local L = LibStub('AceLocale-3.0'):GetLocale('AutoRoller', true)

--  db[info[#info]]

function self:OptionGet(info)
   return self.db.profile[info[#info]]
end

function self:OptionSet(info, value)
   self.db.profile[info[#info]] = value
end

function self:RuleGet(info)
   return self.db.profile.rules[self.ruleNameLUT[info[#info-1]]][info[#info]]
end

function self:RuleSet(info, value)
   self.db.profile.rules[self.ruleNameLUT[info[#info-1]]][info[#info]] = value
end

function self:DeleteRule(info)
   local rules = self.db.profile.rules
   local lut = self.ruleNameLUT

   self.optionTable.args.rules.args[lut[#rules]] = nil --remove last rule

   tremove(rules, lut[info[#info-1]])
end

function self:MoveRuleUp(info)
   --only have to swap rules in db, optionsTable will automatically adjust
   local n = self.ruleNameLUT[info[#info-1]]

   if n == 1 then return end

   local rules = self.db.profile.rules

   rules[n],rules[n-1] = rules[n-1],rules[n]
end

function self:NewRule()
   local rules, lut = self.db.profile.rules, self.ruleNameLUT
   local n = #rules+1
   local name = 'rule'..n
   lut[n], lut[name] = name, n
   rules[n] = {
      enabled = false,
      action = 3,
      logic = "",
   }
   self:BuildRuleOption(n, name)
end

self.optionTable = {
   name = 'AutoRoller',
   desc = L["Automatically roll on items."],
   type = 'group',
   handler = self,
   get = 'OptionGet',
   set = 'OptionSet',
   args = {
      desc = {
         name = L["Automatically roll on items."],
         type = 'description',
         order = 100,
         width = 'full',
      },
      enabled = {
         name = ENABLE,
         type = 'toggle',
         order = 200,
         get = 'IsEnabled',
         width = 'full',
         set = function (info, v)
            local self = info.handler
            self.db.profile.enabled = v
            if v then
               self:Enable()
            else
               self:Disable()
            end
         end,
      },
      chat = {
         name = CHAT_OPTIONS_LABEL,
         desc = L["Output to chat when automatically rolling on an item."],
         type = 'toggle',
         order = 300,
         width = 'full',
      },
      disableBoPConfirm = {
         name = L["Hide BoP Confirmation"],
         desc = L["Hide the confimation dialog that appears every time you roll on a bind on pickup item."],
         type = 'toggle',
         order = 350,
         width = 'full',
      },
      newRule = {
         name = ADD_FILTER,
         type = 'execute',
         order = 400,
         func = 'NewRule',
      },
      rules = {
         name = FILTERS,
         type = 'group',
         inline = true,
         order = 500,
         set = 'RuleSet',
         get = 'RuleGet',
         func = 'RuleFunc',
         args = {
            --rules go here
         },
      },
      help = {
         name = L["TagHelp"],
         type = 'description',
         order = 600,
         width = 'full',
      }
   }
}

local ruleArgs = {
   enabled = {
      name = ENABLE,
      type = 'toggle',
      order = 100,
      width = 'half',
   },
   action = {
      name = L["Action"],
      type = 'select',
      values = self.rollType,
      order = 101,
   },
   delete = {
      name = DELETE,
      type = 'execute',
      order = 201,
      width = 'half',
      confirm = true,
      func = 'DeleteRule',
      confirmText = CONFIRM_COMBAT_FILTER_DELETE,
   },
   moveup = {
      name = MOVE_FILTER_UP,
      type = 'execute',
      order = 202,
      func = 'MoveRuleUp',
   },
   logic = {
      name = L["Logic"],
      type = 'input',
      order = 300,
      width = 'full',
      validate = 'ValidateLogic',
   },
}

function self:BuildRuleOption(n, name)
   local args = self.optionTable.args.rules.args

   args[name] = {
      type = 'group',
      name = FILTER..n,
      order = 100+n,
      args = ruleArgs,
   }
end

function self:RegisterOptions()
   local options = self.optionTable
   local profile = LibStub('AceDBOptions-3.0'):GetOptionsTable(self.db)

   local registry = LibStub('AceConfigRegistry-3.0')
   local dialog = LibStub('AceConfigDialog-3.0')
   LibStub('AceConfig-3.0'):RegisterOptionsTable('AutoRoller', options, 'autoroller')

	registry:RegisterOptionsTable('AutoRoller Options', options)
	registry:RegisterOptionsTable('AutoRoller Profiles', profile)

   local main = dialog:AddToBlizOptions('AutoRoller Options', 'AutoRoller')
   self.optionsFrames = {
      main = main,
		profiles = dialog:AddToBlizOptions('AutoRoller Profiles', 'Profiles', 'AutoRoller'),
   }

   if self.dataobj then
      self.dataobj.OnClick = function() InterfaceOptionsFrame_OpenToCategory(main) end
   end
end
